#pragma clang diagnostic push
#pragma ide diagnostic ignored "performance-unnecessary-value-param"
#pragma ide diagnostic ignored "readability-convert-member-functions-to-static"

#include "controllers.h"
#include "dto/dto.h"

namespace api {
    void admin::users(req_type, cbk_type cbk) {
        db::MapperUser user_mapper(drogon::app().getDbClient());
        std::vector<dto::Responses::GetUsersEntry> entries;
        auto users = user_mapper.findAll();
        for (const db::User& user : users)
            entries.emplace_back(
                    user.getValueOfId(),
                    user.getValueOfGitlab() != 0,
                    db::User_getEnumTfaType(user) != db::tfaTypes::NONE,
                    user.getValueOfName(),
                    db::User_getEnumRole(user)
            );
        cbk(dto::Responses::get_admin_users_res(entries));
    }

    void admin::set_role(req_type req, cbk_type cbk) {
        Json::Value& json = *req->jsonObject();
        try {
            uint64_t user_id = dto::json_get<uint64_t>(json, "user").value();
            db::UserRole role = (db::UserRole)dto::json_get<int>(json, "role").value();

            db::MapperUser user_mapper(drogon::app().getDbClient());
            auto user = user_mapper.findByPrimaryKey(user_id);
            user.setRole(role);
            user_mapper.update(user);

            cbk(dto::Responses::get_success_res());
        } catch (const std::exception&) {
            cbk(dto::Responses::get_badreq_res("Validation error"));
        }
    }

    void admin::logout(req_type req, cbk_type cbk) {
        Json::Value& json = *req->jsonObject();
        try {
            uint64_t user_id = dto::json_get<uint64_t>(json, "user").value();

            db::MapperUser user_mapper(drogon::app().getDbClient());
            auto user = user_mapper.findByPrimaryKey(user_id);
            auth::revoke_all(user);

            cbk(dto::Responses::get_success_res());
        } catch (const std::exception&) {
            cbk(dto::Responses::get_badreq_res("Validation error"));
        }
    }

    void admin::delete_user(req_type req, cbk_type cbk) {
        Json::Value& json = *req->jsonObject();
        msd::channel<std::string> chan;
        try {
            uint64_t user_id = dto::json_get<uint64_t>(json, "user").value();

            db::MapperUser user_mapper(drogon::app().getDbClient());
            auto user = user_mapper.findByPrimaryKey(user_id);
            auth::revoke_all(user);

            std::unique_lock lock(*fs::get_user_mutex(user.getValueOfId()));

            fs::delete_node(fs::get_node(user.getValueOfRootId()).value(), chan, true);
            user_mapper.deleteOne(user);
            cbk(dto::Responses::get_success_res());
        } catch (const std::exception&) {
            cbk(dto::Responses::get_badreq_res("Validation error"));
        }
    }

    void admin::disable_2fa(req_type req, cbk_type cbk) {
        Json::Value& json = *req->jsonObject();
        try {
            uint64_t user_id = dto::json_get<uint64_t>(json, "user").value();

            db::MapperUser user_mapper(drogon::app().getDbClient());
            auto user = user_mapper.findByPrimaryKey(user_id);
            user.setTfaType(db::tfaTypes::NONE);
            user_mapper.update(user);

            cbk(dto::Responses::get_success_res());
        } catch (const std::exception&) {
            cbk(dto::Responses::get_badreq_res("Validation error"));
        }
    }
}
#pragma clang diagnostic pop